@@ -22,10 +22,10 @@
 #include "mysqlx_session.h"
 #include "util/allocator.h"
 #include "util/exceptions.h"
+#include "util/functions.h"
 #include "util/json_utils.h"
 #include "util/object.h"
 #include "util/string_utils.h"
-#include "util/zend_utils.h"
 #include <chrono>
 #include <condition_variable>
 #include <mutex>
@@ -228,9 +228,11 @@ public:
 private:
 	bool is_full() const;
 	bool has_idle_connection() const;
+	bool can_pool_connection(drv::XMYSQLND_SESSION closing_connection) const;
 	drv::XMYSQLND_SESSION create_new_connection(const bool persistent);
 	drv::XMYSQLND_SESSION create_non_pooled_connection();
 	drv::XMYSQLND_SESSION create_idle_connection(drv::XMYSQLND_SESSION closing_connection);
+	void push_idle_connection(drv::XMYSQLND_SESSION closing_connection);
 	drv::XMYSQLND_SESSION add_new_connection();
 	drv::XMYSQLND_SESSION pop_idle_connection();
 	drv::XMYSQLND_SESSION try_pop_idle_connection(std::unique_lock<std::mutex>& lck);
@@ -242,7 +244,7 @@ private:
 
 private:
 	//Connection_pool_callback
-	virtual void on_close(drv::XMYSQLND_SESSION closing_connection);
+	void on_close(drv::XMYSQLND_SESSION closing_connection) override;
 
 private:
 	std::mutex mtx;
@@ -339,6 +341,12 @@ bool Connection_pool::has_idle_connection() const
 	return !idle_connections.empty();
 }
 
+bool Connection_pool::can_pool_connection(drv::XMYSQLND_SESSION closing_connection) const
+{
+	const auto& state = closing_connection->get_data()->state;
+	return !state.has_closed_with_error();
+}
+
 drv::XMYSQLND_SESSION Connection_pool::create_new_connection(const bool persistent)
 {
 	drv::XMYSQLND_SESSION connection{ drv::create_session(persistent) };
@@ -358,10 +366,18 @@ drv::XMYSQLND_SESSION Connection_pool::create_idle_connection(
 {
 	drv::XMYSQLND_SESSION idle_connection{
 		std::make_shared<drv::xmysqlnd_session>(std::move(*closing_connection)) };
-	closing_connection->get_data()->state.set(drv::SESSION_CLOSED);
+	closing_connection->get_data()->state.set_closed(drv::Session_close_reason::None);
 	return idle_connection;
 }
 
+void Connection_pool::push_idle_connection(
+	drv::XMYSQLND_SESSION closing_connection)
+{
+	drv::XMYSQLND_SESSION idle_connection{ create_idle_connection(closing_connection) };
+	idle_connections.push_back({ idle_connection, max_idle_time });
+	on_idle_connection_added.notify_one();
+}
+
 drv::XMYSQLND_SESSION Connection_pool::add_new_connection()
 {
 	drv::XMYSQLND_SESSION connection{ create_new_connection(true) };
@@ -404,7 +420,7 @@ bool Connection_pool::wait_for_idle_connection(std::unique_lock<std::mutex>& lck
 void Connection_pool::close_active_connections()
 {
 	for (auto& conn : active_connections) {
-		conn->close(drv::SESSION_CLOSE_EXPLICIT);
+		conn->close(drv::Session_close_reason::Explicit);
 	}
 	active_connections.clear();
 }
@@ -412,7 +428,7 @@ void Connection_pool::close_active_connections()
 void Connection_pool::close_idle_connections()
 {
 	for (auto& conn : idle_connections) {
-		conn.connection->close(drv::SESSION_CLOSE_EXPLICIT);
+		conn.connection->close(drv::Session_close_reason::Explicit);
 	}
 	idle_connections.clear();
 }
@@ -421,11 +437,13 @@ void Connection_pool::on_close(drv::XMYSQLND_SESSION closing_connection)
 {
 	std::lock_guard<std::mutex> lck(mtx);
 	auto it{ active_connections.find(closing_connection) };
-	if (it != active_connections.end()) {
-		active_connections.erase(it);
-		drv::XMYSQLND_SESSION idle_connection{ create_idle_connection(closing_connection) };
-		idle_connections.push_back({ idle_connection, max_idle_time });
-		on_idle_connection_added.notify_one();
+	if (it == active_connections.end()) {
+		return; // connection wasn't in pool
+	}
+
+	active_connections.erase(it);
+	if (can_pool_connection(closing_connection)) {
+		push_idle_connection(closing_connection);
 	}
 }
 
@@ -532,7 +550,7 @@ struct Client_data : public util::custom_allocable
 	shared_client_state state;
 };
 
-Connection_pool& fetch_connection_pool(zval* from)
+Connection_pool& fetch_connection_pool(util::raw_zval* from)
 {
 	auto& data_object{ util::fetch_data_object<Client_data>(from) };
 	return data_object.state->conn_pool;
@@ -558,8 +576,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_client, getSession)
 {
 	DBG_ENTER("mysqlx_client::getSession");
 
-	zval* object_zv{nullptr};
-	if (util::zend::parse_method_parameters(
+	util::raw_zval* object_zv{nullptr};
+	if (util::get_method_arguments(
 		execute_data, getThis(),
 		"O", &object_zv, client_class_entry) == FAILURE) {
 		DBG_VOID_RETURN;
@@ -567,7 +585,7 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_client, getSession)
 
 	auto& conn_pool{ fetch_connection_pool(object_zv) };
 	drv::XMYSQLND_SESSION connection{ conn_pool.get_connection() };
-	mysqlx_new_session(return_value, connection);
+	create_session(connection).move_to(return_value);
 
 	DBG_VOID_RETURN;
 }
@@ -576,8 +594,8 @@ MYSQL_XDEVAPI_PHP_METHOD(mysqlx_client, close)
 {
 	DBG_ENTER("mysqlx_client::close");
 
-	zval* object_zv{nullptr};
-	if (util::zend::parse_method_parameters(
+	util::raw_zval* object_zv{nullptr};
+	if (util::get_method_arguments(
 		execute_data, getThis(),
 		"O", &object_zv, client_class_entry) == FAILURE) {
 		DBG_VOID_RETURN;
@@ -622,19 +640,19 @@ client_object_allocator(zend_class_entry* class_type)
 	DBG_RETURN(&mysqlx_object->zo);
 }
 
-void
-mysqlx_new_client(
+util::zvalue
+create_client(
 	const util::string_view& connection_uri,
-	const util::string_view& client_options_desc,
-	zval* return_value)
+	const util::string_view& client_options_desc)
 {
-	DBG_ENTER("mysqlx_new_client");
+	DBG_ENTER("create_client");
 
-	Client_data& data_object{ util::init_object<Client_data>(client_class_entry, return_value) };
+	util::zvalue client_obj;
+	Client_data& client_data{ util::init_object<Client_data>(client_class_entry, client_obj) };
 	Client_state_manager& csm{ Client_state_manager::get() };
-	data_object.state = csm.get_client_state(std::string{ connection_uri }, client_options_desc);
+	client_data.state = csm.get_client_state(std::string{ connection_uri }, client_options_desc);
 
-	DBG_VOID_RETURN;
+	DBG_RETURN(client_obj);
 }
 
 } // anonymous namespace
@@ -664,13 +682,13 @@ mysqlx_unregister_client_class(UNUSED_SHUTDOWN_FUNC_ARGS)
 
 MYSQL_XDEVAPI_PHP_FUNCTION(mysql_xdevapi_getClient)
 {
-	util::param_string connection_uri;
-	util::param_string client_options_desc;
+	util::arg_string connection_uri;
+	util::arg_string client_options_desc;
 
 	RETVAL_NULL();
 
 	DBG_ENTER("mysql_xdevapi_getClient");
-	if (FAILURE == util::zend::parse_function_parameters(execute_data, "s|s",
+	if (FAILURE == util::get_function_arguments(execute_data, "s|s",
 		&connection_uri.str, &connection_uri.len,
 		&client_options_desc.str, &client_options_desc.len))
 	{
@@ -680,7 +698,7 @@ MYSQL_XDEVAPI_PHP_FUNCTION(mysql_xdevapi_getClient)
 	#if PHP_DEBUG
 	drv::verify_connection_string(connection_uri.to_string());
 	#endif
-	mysqlx_new_client(connection_uri.to_view(), client_options_desc.to_view(), return_value);
+	create_client(connection_uri.to_view(), client_options_desc.to_view()).move_to(return_value);
 
 	DBG_VOID_RETURN;
 }
